package com.jeecms.utils.resource.operator;

import org.slf4j.LoggerFactory;
import com.jeecms.utils.resource.ResourceUtil;
import com.jeecms.utils.DownloadUtil;
import java.io.File;
import org.springframework.core.io.FileSystemResource;
import com.jeecms.utils.resource.store.IResourceStore;
import com.jeecms.utils.URLUtil;
import com.jeecms.utils.resource.Config;
import java.io.Serializable;
import java.util.List;
import com.jeecms.utils.resource.resolve.ResType;
import com.jeecms.utils.resource.interceptor.ResourceInterceptor;
import java.io.IOException;
import java.util.function.Consumer;
import org.springframework.core.io.Resource;
import org.slf4j.Logger;

public abstract class AbstractResourceOperator implements ResourceOperator
{
    private static final Logger log;
    
    public UploadResult upload(final Resource resource, final Consumer<UploadContext> consumer) throws IOException {
        final UploadContext context = UploadContext.wrap(resource);
        int interceptorIdx = 0;
        UploadResult uploadResult = null;
        final List<ResourceInterceptor> interceptors = context.getConfig().getInterceptors();
        try {
            this.openClient(context);
        }
        catch (final Exception e) {
            if (AbstractResourceOperator.log.isDebugEnabled()) {
                AbstractResourceOperator.log.debug("资源上传开启客户端失败：{}", context.getResourceOperator().getClass());
            }
            context.setException(e);
            try {
                this.closeClient(context);
            }
            catch (final Exception ex) {}
            IOException ioException;
            if (e instanceof IOException) {
                ioException = (IOException)e;
            }
            else {
                ioException = new IOException(e);
            }
            throw ioException;
        }
        try {
            if (consumer != null) {
                consumer.accept(context);
            }
            try {
                while (interceptorIdx < interceptors.size()) {
                    if (AbstractResourceOperator.log.isDebugEnabled()) {
                        AbstractResourceOperator.log.debug("资源上传拦截器前置处理：idx {}, interceptor {}", (Object)interceptorIdx, interceptors.get(interceptorIdx).getClass());
                    }
                    interceptors.get(interceptorIdx).preHandler(context);
                    ++interceptorIdx;
                }
            }
            catch (final Exception e) {
                context.setException(e);
                AbstractResourceOperator.log.warn("资源上传拦截器前置处理异常：" + interceptors.get(interceptorIdx).getClass().getName(), e);
            }
            if (!context.hasException()) {
                try {
                    uploadResult = this.doUpload(context);
                    uploadResult.setLength(context.getContentLength());
                    uploadResult.setResType((ResType)context.getAttach("RES_TYPE"));
                    context.setUploadResult(uploadResult);
                    if (AbstractResourceOperator.log.isDebugEnabled()) {
                        AbstractResourceOperator.log.debug("资源上传成功：文件名[{}],  实际上传名[{}]", resource.getFilename(), context.getFilename());
                    }
                }
                catch (final Exception e) {
                    context.setException(e);
                }
            }
            --interceptorIdx;
            while (interceptorIdx >= 0) {
                try {
                    if (AbstractResourceOperator.log.isDebugEnabled()) {
                        AbstractResourceOperator.log.debug("资源上传拦截器后置处理：idx {}, interceptor {}", (Object)interceptorIdx, interceptors.get(interceptorIdx).getClass());
                    }
                    interceptors.get(interceptorIdx).postHandler(context);
                }
                catch (final Exception e) {
                    context.setException(e);
                    AbstractResourceOperator.log.warn("资源上传拦截器后置处理异常：" + interceptors.get(interceptorIdx).getClass().getName(), e);
                }
                --interceptorIdx;
            }
            if (context.hasException()) {
                final Exception e = context.getException();
                IOException ioException;
                if (e instanceof IOException) {
                    ioException = (IOException)e;
                }
                else {
                    if (e instanceof RuntimeException) {
                        throw (RuntimeException)e;
                    }
                    ioException = new IOException(e);
                }
                throw ioException;
            }
            if (AbstractResourceOperator.log.isDebugEnabled()) {
                AbstractResourceOperator.log.debug("上传结果：{}", uploadResult);
            }
            return uploadResult;
        }
        finally {
            this.closeClient(context);
        }
    }
    
    protected void openClient(final UploadContext context) throws IOException {
    }
    
    protected void closeClient(final UploadContext context) throws IOException {
    }
    
    protected abstract UploadResult doUpload(final UploadContext context) throws IOException;
    
    public boolean delete(Serializable mark) {
        final Config config = Config.getInstance();
        final IResourceStore resourceStore = config.getResourceStore();
        if (mark != null && mark instanceof Long) {
            mark = this.getUrlById(resourceStore, (Long)mark);
        }
        if (mark != null && mark instanceof String) {
            String strMark = (String)mark;
            if (strMark.startsWith("http")) {
                strMark = URLUtil.getPath(strMark);
            }
            if (strMark.startsWith("/")) {
                if (mark instanceof Long) {
                    resourceStore.remove(mark);
                }
                else {
                    resourceStore.remove((Serializable)strMark);
                }
                try {
                    final boolean delete = this.doDelete(this.clearPath(strMark));
                    return delete;
                }
                catch (final IOException e) {
                    AbstractResourceOperator.log.info("删除资源[{}]失败", mark, e);
                    return false;
                }
            }
        }
        throw new IllegalArgumentException("无法根据mark[" + mark + "]删除资源");
    }
    
    protected abstract boolean doDelete(final String path) throws IOException;
    
    public UploadResult copy(Serializable mark, final String dir) throws IOException {
        final Config config = Config.getInstance();
        final IResourceStore resourceStore = config.getResourceStore();
        if (mark instanceof Long) {
            mark = this.getUrlById(resourceStore, (Long)mark);
        }
        if (mark instanceof String) {
            String strMark = (String)mark;
            if (strMark.startsWith("http")) {
                strMark = URLUtil.getPath(strMark);
            }
            if (strMark.startsWith("/")) {
                return this.doCopy(this.clearPath(strMark), this.unifyDir(dir));
            }
        }
        throw new IllegalArgumentException("无法根据mark[" + mark + "]复制资源");
    }
    
    public UploadResult doCopy(final String path, final String dir) throws IOException {
        final File file = this.doDownload(path, dir);
        final Resource resource = (Resource)new FileSystemResource(file);
        final UploadResult uploadResult = this.upload(resource, context -> context.setDir(dir));
        return uploadResult;
    }
    
    public File download(Serializable mark, final String dir) throws IOException {
        final Config config = Config.getInstance();
        final IResourceStore resourceStore = config.getResourceStore();
        if (mark != null && mark instanceof Long) {
            mark = this.getUrlById(resourceStore, (Long)mark);
        }
        if (mark != null && mark instanceof String) {
            final String strMark = (String)mark;
            if (strMark.startsWith("http")) {
                return DownloadUtil.download(strMark, dir);
            }
            if (strMark.startsWith("/")) {
                return this.doDownload(this.clearPath(strMark), dir);
            }
        }
        throw new IllegalArgumentException("无法根据mark[" + mark + "]下载资源");
    }
    
    public File doDownload(final String path, final String dir) throws IOException {
        return DownloadUtil.download(ResourceUtil.accessUrl((Serializable)path), dir);
    }
    
    public UploadResult move(Serializable mark, final String dir) throws IOException {
        final Config config = Config.getInstance();
        final IResourceStore resourceStore = config.getResourceStore();
        if (mark != null && mark instanceof Long) {
            mark = this.getUrlById(resourceStore, (Long)mark);
        }
        if (mark != null && mark instanceof String) {
            String strMark = (String)mark;
            if (strMark.startsWith("http")) {
                strMark = URLUtil.getPath(strMark);
            }
            if (strMark.startsWith("/")) {
                return this.doMove(this.clearPath(strMark), this.unifyDir(dir));
            }
        }
        throw new IllegalArgumentException("无法根据mark[" + mark + "]移动资源");
    }
    
    public UploadResult doMove(final String path, final String dir) throws IOException {
        final UploadResult uploadResult = this.doCopy(path, dir);
        this.doDelete(path);
        return uploadResult;
    }
    
    public boolean exist(final UploadContext context) throws IOException {
        return false;
    }
    
    public String getAlias(final String originFileName, final Long siteId, final Long groupId, final Integer originType) throws IOException {
        final Config config = Config.getInstance();
        final IResourceStore resourceStore = config.getResourceStore();
        return resourceStore.getAlias(originFileName, siteId, groupId, originType);
    }
    
    private String getUrlById(final IResourceStore resourceStore, final Long id) {
        if (resourceStore == null) {
            throw new IllegalArgumentException("未配置资源库，无法根据id查找资源");
        }
        final UploadResult uploadResult = resourceStore.getUploadResultById(id);
        if (uploadResult == null) {
            throw new IllegalArgumentException("资源库中找不到资源：id [" + id + "]");
        }
        return uploadResult.getUrl();
    }
    
    private String clearPath(final String path) {
        final int questionMarkIndex = path.indexOf(63);
        if (questionMarkIndex != -1) {
            return path.substring(0, questionMarkIndex);
        }
        return path;
    }
    
    private String unifyDir(String dir) {
        dir = dir.replaceAll("\\\\+", "/");
        if (!dir.startsWith("/")) {
            dir = "/" + dir;
        }
        if (!dir.endsWith("/")) {
            dir += "/";
        }
        return dir;
    }
    
    static {
        log = LoggerFactory.getLogger(AbstractResourceOperator.class);
    }
}
